use std::
{
    time::Instant,
};

pub enum MsgFromController
{
	RepaintGui (RgbaFrame),
	StartCapture,
	StartJpegDecoder (JpegFrame),
}

#[derive (Clone)]
pub struct JpegFrame
{
    pub data: Vec <u8>,
    pub capture_time: Instant,
}

#[derive (Clone)]
pub struct RgbaFrame
{
    pub data: Vec <u8>,
    pub capture_time: Instant,
}

pub struct Controller
{
    gui_has_new_frame: bool,
    gui_needs_frame: bool,
    
    capture_is_running: bool,
    
    jpeg_decoder_is_running: bool,
    jpeg_decoder_has_new_frame: bool,
    jpeg_decoder_needs_frame: bool,
    
    rgba_frame: RgbaFrame,
    jpeg_frame: JpegFrame,
}

impl Controller
{
    pub fn new (now: Instant) -> Self
    {
        Self
        {
            gui_has_new_frame: false,
            gui_needs_frame: false,
            
            capture_is_running: false,
            jpeg_decoder_has_new_frame: false,
            jpeg_decoder_is_running: false,
            jpeg_decoder_needs_frame: false,
            
            rgba_frame: RgbaFrame
            {
                data: Default::default (), 
                capture_time: now,
            },
            jpeg_frame: JpegFrame
            {
                data: Default::default (),
                capture_time: now,
            },
        }
    }
    
	pub fn handle_rgba_frame (&mut self, frame: RgbaFrame)
	{
        // println! ("handle_rgba_frame");
        self.gui_has_new_frame = true;
        self.jpeg_decoder_is_running = false;
        self.jpeg_decoder_has_new_frame = false;
		self.rgba_frame = frame;
	}
	
	pub fn handle_capture (&mut self, jpeg: JpegFrame)
	{
        // println! ("handle_capture");
        self.capture_is_running = false;
		self.jpeg_frame = jpeg;
		self.jpeg_decoder_has_new_frame = true;
	}
	
	pub fn handle_gui_needs_frame (&mut self)
	{
        // println! ("handle_gui_needs_frame");
		self.gui_needs_frame = true;
	}
	
	pub fn poll (&mut self) -> Option <MsgFromController>
	{
		if self.gui_needs_frame && self.gui_has_new_frame
		{
            self.gui_has_new_frame = false;
            self.gui_needs_frame = false;
            // println! ("RepaintGui");
			Some (MsgFromController::RepaintGui (self.rgba_frame.clone ()))
		}
		else if self.jpeg_decoder_has_new_frame && ! self.jpeg_decoder_is_running && self.jpeg_decoder_needs_frame
		{
			self.jpeg_decoder_is_running = true;
			self.jpeg_decoder_needs_frame = false;
			// println! ("StartJpegDecoder");
			Some (MsgFromController::StartJpegDecoder (self.jpeg_frame.clone ()))
		}
		else if self.jpeg_decoder_needs_frame && ! self.jpeg_decoder_has_new_frame && ! self.capture_is_running
		{
            self.capture_is_running = true;
            // println! ("StartCapture");
			Some (MsgFromController::StartCapture)
		}
		else if self.gui_needs_frame && ! self.gui_has_new_frame
		{
			self.jpeg_decoder_needs_frame = true;
			None
		}
		else
		{
			None
		}
	}
}
